@@ -352,6 +352,7 @@ public class ASB_impl extends Resource_ImplBase implements ASB {
   /**
    * Gets the MBean that provides the management interface to this AE. Returns the same object as
    * UimaContext.getManagementInterface() but casted to the AnalysisEngineManagement type.
+   * @return the MBean for the management interface to this AE
    */
   protected AnalysisEngineManagementImpl getMBean() {
     return (AnalysisEngineManagementImpl) mAggregateUimaContext.getManagementInterface();
@@ -505,7 +506,7 @@ public class ASB_impl extends Resource_ImplBase implements ASB {
             if (casIteratorStack.isEmpty()) {
               return null; // there are no more CAS Iterators to obtain CASes from
             }
-            StackFrame frame = (StackFrame) casIteratorStack.peek();
+            StackFrame frame = casIteratorStack.peek();
             try {
               if (frame.casIterator.hasNext()) {
                 cas = frame.casIterator.next();
@@ -553,7 +554,7 @@ public class ASB_impl extends Resource_ImplBase implements ASB {
             //Simple Step
             if (nextStep instanceof SimpleStep) {
               String nextAeKey = ((SimpleStep) nextStep).getAnalysisEngineKey();
-              AnalysisEngine nextAe = (AnalysisEngine) mComponentAnalysisEngineMap.get(nextAeKey);
+              AnalysisEngine nextAe = mComponentAnalysisEngineMap.get(nextAeKey);
               if (nextAe != null) {
                 //check if we have to set result spec, to support capability language flow
                 if (nextStep instanceof SimpleStepWithResultSpec) {
@@ -612,7 +613,7 @@ public class ASB_impl extends Resource_ImplBase implements ASB {
                 String nextAeKey = destinations.get(0);
                 destinations.remove(0); 
                 //execute this step as we would a single step
-                AnalysisEngine nextAe = (AnalysisEngine) mComponentAnalysisEngineMap.get(nextAeKey);
+                AnalysisEngine nextAe = mComponentAnalysisEngineMap.get(nextAeKey);
                 if (nextAe != null) {
                   // invoke next AE in flow
                   CasIterator casIter = null;
