@@ -21,15 +21,17 @@ package org.apache.uima.analysis_engine.impl;
 
 import java.text.DecimalFormat;
 import java.util.Collections;
-import java.util.HashSet;
 import java.util.Iterator;
 import java.util.LinkedHashMap;
 import java.util.Map;
-import java.util.Set;
+import java.util.concurrent.Callable;
+import java.util.concurrent.atomic.AtomicInteger;
+import java.util.concurrent.atomic.AtomicLong;
 import java.util.regex.Pattern;
 
 import org.apache.uima.UimaContextAdmin;
 import org.apache.uima.analysis_engine.AnalysisEngineManagement;
+import org.apache.uima.util.ConcurrentHashMapWithProducer;
 
 /**
  * Implements Monitoring/Management interface to an AnalysisEngine.
@@ -48,29 +50,29 @@ public class AnalysisEngineManagementImpl
    * This static set is needed to keep track of what names we've already used for "root" MBeans
    * (those representing top-level AEs and CPEs).
    */
-  private static Set<String> usedRootNames = new HashSet<String>();
+  private final static ConcurrentHashMapWithProducer<String, AtomicInteger> usedRootNames = new ConcurrentHashMapWithProducer<String, AtomicInteger>();
 
   private String name;
 
-  private long numProcessed;
+  private final AtomicLong numProcessed = new AtomicLong(0);
 
-  private long markedAnalysisTime;
+  private final AtomicLong markedAnalysisTime = new AtomicLong(0);
 
-  private long markedBatchProcessCompleteTime;
+  private final AtomicLong markedBatchProcessCompleteTime = new AtomicLong(0);
 
-  private long markedCollectionProcessCompleteTime;
+  private final AtomicLong markedCollectionProcessCompleteTime = new AtomicLong(0);
 
-  private long markedServiceCallTime;
+  private final AtomicLong markedServiceCallTime = new AtomicLong(0);
 
-  private long analysisTime;
+  private final AtomicLong analysisTime = new AtomicLong(0);
 
-  private long batchProcessCompleteTime;
+  private final AtomicLong batchProcessCompleteTime = new AtomicLong(0);
 
-  private long collectionProcessCompleteTime;
+  private final AtomicLong collectionProcessCompleteTime = new AtomicLong(0);
 
-  private long serviceCallTime;
+  private final AtomicLong serviceCallTime = new AtomicLong(0);
 
-  private Map<String, AnalysisEngineManagement> components = new LinkedHashMap<String, AnalysisEngineManagement>();
+  private final Map<String, AnalysisEngineManagement> components = Collections.synchronizedMap(new LinkedHashMap<String, AnalysisEngineManagement>());
 
   private String uniqueMBeanName;
 
@@ -101,54 +103,52 @@ public class AnalysisEngineManagementImpl
 
   
   public void reportAnalysisTime(long time) {
-    analysisTime += time;
+    analysisTime.addAndGet(time);
   }
 
   public void reportBatchProcessCompleteTime(long time) {
-    batchProcessCompleteTime += time;
+    batchProcessCompleteTime.addAndGet(time);
   }
 
   public void reportCollectionProcessCompleteTime(long time) {
-    collectionProcessCompleteTime += time;
+    collectionProcessCompleteTime.addAndGet(time);
   }
 
   public void reportServiceCallTime(long time) {
-    serviceCallTime += time;
+    serviceCallTime.addAndGet(time);
   }
 
   public void incrementCASesProcessed() {
-    numProcessed++;
+    numProcessed.incrementAndGet();
   }
 
   public long getBatchProcessCompleteTime() {
-    return batchProcessCompleteTime;
+    return batchProcessCompleteTime.get();
   }
 
   public long getCollectionProcessCompleteTime() {
-    return collectionProcessCompleteTime;
+    return collectionProcessCompleteTime.get();
   }
 
   public long getAnalysisTime() {
-  	return analysisTime + serviceCallTime;
+  	return analysisTime.get() + serviceCallTime.get();
   }
 
   public long getServiceCallTime() {
-    return serviceCallTime;
+    return serviceCallTime.get();
   }
 
   /**
    * Internal use only. Used to implement backwards compatibility with the ProcessTrace interface.
    */
   public void mark() {
-    markedAnalysisTime = analysisTime;
-    markedBatchProcessCompleteTime = batchProcessCompleteTime;
-    markedCollectionProcessCompleteTime = collectionProcessCompleteTime;
-    markedServiceCallTime = serviceCallTime;
+    markedAnalysisTime.set(analysisTime.get());
+    markedBatchProcessCompleteTime.set(batchProcessCompleteTime.get());
+    markedCollectionProcessCompleteTime.set(collectionProcessCompleteTime.get());
+    markedServiceCallTime.set(serviceCallTime.get());
     // mark components also
-    Iterator<AnalysisEngineManagement> iter = components.values().iterator();
-    while (iter.hasNext()) {
-      AnalysisEngineManagementImpl component = (AnalysisEngineManagementImpl) iter.next();
-      component.mark();
+    for (AnalysisEngineManagement component : components.values()) {
+      ((AnalysisEngineManagementImpl)component).mark();
     }
   }
 
@@ -157,7 +157,7 @@ public class AnalysisEngineManagementImpl
    * @return Batch Process Complete time since mark
    */
   public long getBatchProcessCompleteTimeSinceMark() {
-    return batchProcessCompleteTime - markedBatchProcessCompleteTime;
+    return batchProcessCompleteTime.get() - markedBatchProcessCompleteTime.get();
   }
 
   /**
@@ -165,7 +165,7 @@ public class AnalysisEngineManagementImpl
    * @return Collection Process Complete time since mark
    */
   public long getCollectionProcessCompleteTimeSinceMark() {
-    return collectionProcessCompleteTime - markedCollectionProcessCompleteTime;
+    return collectionProcessCompleteTime.get() - markedCollectionProcessCompleteTime.get();
   }
 
   /**
@@ -173,7 +173,7 @@ public class AnalysisEngineManagementImpl
    * @return Analysis time since mark
    */
   public long getAnalysisTimeSinceMark() {
-    return analysisTime - markedAnalysisTime;
+    return analysisTime.get() - markedAnalysisTime.get();
   }
 
   /**
@@ -181,17 +181,18 @@ public class AnalysisEngineManagementImpl
    * @return service call time since mark
    */
   public long getServiceCallTimeSinceMark() {
-    return serviceCallTime - markedServiceCallTime;
+    return serviceCallTime.get() - markedServiceCallTime.get();
   }
 
   public long getNumberOfCASesProcessed() {
-    return numProcessed;
+    return numProcessed.get();
   }
 
   public String getCASesPerSecond() {
-    if (getAnalysisTime() == 0)
+    long analysisTime = getAnalysisTime();
+    if (analysisTime == 0)
       return "0";
-    float docsPerSecond = (float) numProcessed / getAnalysisTime() * 1000;
+    float docsPerSecond = (float) numProcessed.get() / analysisTime * 1000;
     return format.format(docsPerSecond);
   }
 
@@ -212,19 +213,17 @@ public class AnalysisEngineManagementImpl
   }
 
   public void resetStats() {
-    numProcessed = 0;
-    analysisTime = 0;
-    batchProcessCompleteTime = 0;
-    collectionProcessCompleteTime = 0;
-    serviceCallTime = 0;
-    markedAnalysisTime = 0;
-    markedBatchProcessCompleteTime = 0;
-    markedCollectionProcessCompleteTime = 0;
-    markedServiceCallTime = 0;
+    numProcessed.set(0);
+    analysisTime.set(0);
+    batchProcessCompleteTime.set(0);
+    collectionProcessCompleteTime.set(0);
+    serviceCallTime.set(0);
+    markedAnalysisTime.set(0);
+    markedBatchProcessCompleteTime.set(0);
+    markedCollectionProcessCompleteTime.set(0);
+    markedServiceCallTime.set(0);
     // reset components also
-    Iterator<AnalysisEngineManagement> iter = components.values().iterator();
-    while (iter.hasNext()) {
-      AnalysisEngineManagementImpl component = (AnalysisEngineManagementImpl) iter.next();
+    for (AnalysisEngineManagement component : components.values()) {
       component.resetStats();
     }
   }
@@ -255,7 +254,7 @@ public class AnalysisEngineManagementImpl
       prefix = aCustomPrefix;
       if (!prefix.endsWith(":") && !prefix.endsWith(",")) {
         prefix += ",";
-}
+      }
     }
     // compute the unique name   
     // (first get the rootMBean and assign it a unique name if it doesn't already have one)
@@ -263,18 +262,7 @@ public class AnalysisEngineManagementImpl
             .getRootContext().getManagementInterface();
     if (rootMBean.getUniqueMBeanName() == null) {
       // try to find a unique name for the root MBean
-      String baseRootName = rootMBean.getName();
-      if (baseRootName == null) {
-        baseRootName = "CPE"; // CPE's don't currently have names
-      }
-      String rootName = baseRootName;
-      int i = 2;
-      while (usedRootNames.contains(rootName)) {
-        rootName = baseRootName + " " + i++;
-      }
-      usedRootNames.add(rootName);
-      // create a propertly-formatted MBean name, using the specified prefix
-      rootMBean.uniqueMBeanName = prefix + "name=" + escapeValue(rootName);
+      rootMBean.uniqueMBeanName = prefix + "name=" + escapeValue(getRootName(rootMBean.getName()));
     }
 
     if (rootMBean != this) {
@@ -294,6 +282,33 @@ public class AnalysisEngineManagementImpl
       uniqueMBeanName = makeMBeanName(prefix, aContext.getQualifiedContextName().substring(1), 1);
     }
   }
+  
+  static private final Callable<AtomicInteger> produceAtomicInteger = new Callable<AtomicInteger>() {  
+    @Override
+    public AtomicInteger call() throws Exception{
+      return new AtomicInteger(1);
+    }
+  };
+  
+  /**
+   * 
+   * @param baseRootName
+   * @return the baseRootName (or CPE if null) plus a suffix 2, 3, ... to be unique
+   */
+  // package private for testing
+  static String getRootName(String baseRootName) {
+    if (baseRootName == null) {
+      baseRootName = "CPE"; // CPE's don't currently have names
+    }
+    AtomicInteger suffix;
+    try {
+      suffix = usedRootNames.get(baseRootName, produceAtomicInteger);
+    } catch (Exception e) {
+      throw new RuntimeException(e); // never happen.
+    }
+    int suffixI = suffix.getAndIncrement();
+    return (suffixI == 1) ? baseRootName : baseRootName + suffixI;
+  }
 
   /**
    * Recursive utility method for generating a hierarchical mbean name
