@@ -25,6 +25,7 @@ import java.util.Map;
 import org.apache.uima.Constants;
 import org.apache.uima.UIMAFramework;
 import org.apache.uima.UIMA_IllegalStateException;
+import org.apache.uima.UimaContext;
 import org.apache.uima.UimaContextAdmin;
 import org.apache.uima.UimaContextHolder;
 import org.apache.uima.analysis_component.AnalysisComponent;
@@ -59,9 +60,10 @@ import org.apache.uima.util.Logger;
  */
 public class PrimitiveAnalysisEngine_impl extends AnalysisEngineImplBase implements AnalysisEngine {
   /**
-   * current class
+   * UIMA-5043 Set & restore the UimaContextHolder around calls to user code so it can be used to access the External Settings
    */
   
+  
   private static final Class<PrimitiveAnalysisEngine_impl> CLASS_NAME = PrimitiveAnalysisEngine_impl.class;
  
   /**
@@ -236,7 +238,6 @@ public class PrimitiveAnalysisEngine_impl extends AnalysisEngineImplBase impleme
       return;
     }
 
-    UimaContextHolder.setContext(getUimaContext());  // for use by POJOs
     try {
       Object userObject = annotatorClass.newInstance();
       if (userObject instanceof AnalysisComponent) {
@@ -246,10 +247,9 @@ public class PrimitiveAnalysisEngine_impl extends AnalysisEngineImplBase impleme
                 getAnalysisEngineMetaData(), aAdditionalParams);
       }
     } catch (ResourceInitializationException e) {
-      UimaContextHolder.clearContext();
       throw e;
     } catch (Exception e) {
-      UimaContextHolder.clearContext();
+      
       throw new ResourceInitializationException(
               ResourceInitializationException.COULD_NOT_INSTANTIATE_ANNOTATOR, new Object[] {
                   annotatorClassName, mDescription.getSourceUrlString() }, e);
@@ -262,6 +262,7 @@ public class PrimitiveAnalysisEngine_impl extends AnalysisEngineImplBase impleme
     uimaContext.setLogger(logger);
 
     // initialize AnalysisComponent
+    UimaContext prevContext = UimaContextHolder.setContext(getUimaContext());  // for use by POJOs
     try {
       mAnalysisComponent.initialize(getUimaContext());
       // set up the CAS pool for this AE (this won't do anything if mAnalysisComponent.getCasInstancesRequired() == 0)
@@ -272,7 +273,7 @@ public class PrimitiveAnalysisEngine_impl extends AnalysisEngineImplBase impleme
               ResourceInitializationException.ANNOTATOR_INITIALIZATION_FAILED, new Object[] {
                   annotatorClassName, mDescription.getSourceUrlString() }, e);
     } finally {
-      UimaContextHolder.clearContext();
+      UimaContextHolder.setContext(prevContext);
     }
   }
 
@@ -281,9 +282,12 @@ public class PrimitiveAnalysisEngine_impl extends AnalysisEngineImplBase impleme
    */
   public void destroy() {
     if (mAnalysisComponent != null) {
-      UimaContextHolder.setContext(getUimaContext());  // for use by POJOs
-      mAnalysisComponent.destroy();
-      UimaContextHolder.clearContext();
+      UimaContext prevContext = UimaContextHolder.setContext(getUimaContext());  // for use by POJOs
+      try {
+        mAnalysisComponent.destroy();
+      } finally {
+        UimaContextHolder.setContext(prevContext);
+      }
       getLogger().logrb(Level.CONFIG, CLASS_NAME.getName(), "destroy", LOG_RESOURCE_BUNDLE,
               "UIMA_analysis_engine_destroyed__CONFIG", getMetaData().getName());
     }
@@ -323,22 +327,22 @@ public class PrimitiveAnalysisEngine_impl extends AnalysisEngineImplBase impleme
 
   public void batchProcessComplete() throws AnalysisEngineProcessException {
     enterBatchProcessComplete();
+    UimaContext prevContext = UimaContextHolder.setContext(getUimaContext());  // for use by POJOs
     try {
-      UimaContextHolder.setContext(getUimaContext());  // for use by POJOs
       getAnalysisComponent().batchProcessComplete();
     } finally {
-      UimaContextHolder.clearContext();
+      UimaContextHolder.setContext(prevContext);
       exitBatchProcessComplete();
     }
   }
 
   public void collectionProcessComplete() throws AnalysisEngineProcessException {
     enterCollectionProcessComplete();
+    UimaContext prevContext = UimaContextHolder.setContext(getUimaContext());  // for use by POJOs
     try {
-      UimaContextHolder.setContext(getUimaContext());  // for use by POJOs
       getAnalysisComponent().collectionProcessComplete();
     } finally {
-      UimaContextHolder.clearContext();
+      UimaContextHolder.setContext(prevContext);
       exitCollectionProcessComplete();
     }
   }
@@ -350,7 +354,7 @@ public class PrimitiveAnalysisEngine_impl extends AnalysisEngineImplBase impleme
    *          CAS to be processed by annotator
    * @throws AnalysisEngineProcessException -         
    */
-  protected void callAnalysisComponentProcess(CAS aCAS) throws AnalysisEngineProcessException {
+  protected void callAnalysisComponentProcess(final CAS aCAS) throws AnalysisEngineProcessException {
     // logging and instrumentation
     String resourceName = getMetaData().getName();
     Logger logger = getLogger();
@@ -358,15 +362,16 @@ public class PrimitiveAnalysisEngine_impl extends AnalysisEngineImplBase impleme
             "UIMA_analysis_engine_process_begin__FINE", resourceName);
     try {
       CAS view = null;
+      UimaContext prevContext = UimaContextHolder.setContext(getUimaContext());  // for use by POJOs
       // call Annotator's process method
       try {
 
         // Get the right view of the CAS. Sofa-aware components get the base CAS.
         // Sofa-unaware components get whatever is mapped to the _InitialView.
+        //    note: if initial view is mapped, and the mapped view doesn't exist, throws an exception.
         view = Util.getStartingView(aCAS, mSofaAware, getUimaContextAdmin().getComponentInfo());
         // now get the right interface(e.g. CAS or JCAS)
         // must precede the switchClassLoader call below UIMA-2211
-        UimaContextHolder.setContext(getUimaContext());  // for use by POJOs
         Class<? extends AbstractCas> requiredInterface = mAnalysisComponent.getRequiredCasInterface();
         AbstractCas casToPass = getCasManager().getCasInterface(view, requiredInterface);
 
@@ -403,7 +408,6 @@ public class PrimitiveAnalysisEngine_impl extends AnalysisEngineImplBase impleme
       } catch (Exception e) {
         // catching Throwable to catch out-of-memory errors too, which are not Exceptions
         if (null != view) {
-          view.setCurrentComponentInfo(null);
           ((CASImpl)view).restoreClassLoaderUnlockCas();
         }
         if (e instanceof AnalysisEngineProcessException) {
@@ -414,12 +418,12 @@ public class PrimitiveAnalysisEngine_impl extends AnalysisEngineImplBase impleme
         }
       } catch (Error e) {  // out of memory error, for instance
         if (null != view) {
-          view.setCurrentComponentInfo(null);
           ((CASImpl)view).restoreClassLoaderUnlockCas();
         }
         throw e;
       } finally {
-        UimaContextHolder.clearContext();
+        aCAS.setCurrentComponentInfo(null); // https://issues.apache.org/jira/browse/UIMA-5097
+        UimaContextHolder.setContext(prevContext);
       }
 
       // log end of event
@@ -521,8 +525,8 @@ public class PrimitiveAnalysisEngine_impl extends AnalysisEngineImplBase impleme
    */
   protected CAS callAnalysisComponentNext() throws AnalysisEngineProcessException,
           ResultNotSupportedException {
+    UimaContext prevContext = UimaContextHolder.setContext(getUimaContext());  // for use by POJOs
     try {
-      UimaContextHolder.setContext(getUimaContext());  // for use by POJOs
       AbstractCas absCas = mAnalysisComponent.next();
       getMBean().incrementCASesProcessed();
       // notify UimaContext that a CAS was returned -- it uses
@@ -537,7 +541,7 @@ public class PrimitiveAnalysisEngine_impl extends AnalysisEngineImplBase impleme
       } else {
         casToReturn = (CAS) absCas;
       }
-      casToReturn = casToReturn.getView(CAS.NAME_DEFAULT_SOFA);
+      casToReturn = casToReturn.getView(CAS.NAME_DEFAULT_SOFA); 
 
       // clear the CAS's component info, since it is no longer
       // being processed by this AnalysisComponent
@@ -552,7 +556,7 @@ public class PrimitiveAnalysisEngine_impl extends AnalysisEngineImplBase impleme
       else
         throw new AnalysisEngineProcessException(e);
     } finally {
-      UimaContextHolder.clearContext();
+      UimaContextHolder.setContext(prevContext);
     }
   }
 
@@ -564,13 +568,13 @@ public class PrimitiveAnalysisEngine_impl extends AnalysisEngineImplBase impleme
     super.reconfigure();
 
     // inform the annotator
+    UimaContext prevContext = UimaContextHolder.setContext(getUimaContext());  // for use by POJOs
     try {
-      UimaContextHolder.setContext(getUimaContext());  // for use by POJOs
       mAnalysisComponent.reconfigure();
     } catch (ResourceInitializationException e) {
       throw new ResourceConfigurationException(e);
     } finally {
-      UimaContextHolder.clearContext();
+      UimaContextHolder.setContext(prevContext);
     }
   }
 
@@ -602,8 +606,8 @@ public class PrimitiveAnalysisEngine_impl extends AnalysisEngineImplBase impleme
       if (casAvailable) {
         return true;
       }
+      UimaContext prevContext = UimaContextHolder.setContext(getUimaContext());  // for use by POJOs
       try {
-        UimaContextHolder.setContext(getUimaContext());  // for use by POJOs
         casAvailable = mMyAnalysisComponent.hasNext();
         if (!casAvailable) {
           //when hasNext returns false, by contract the AnalysisComponent is done processing its
@@ -622,7 +626,7 @@ public class PrimitiveAnalysisEngine_impl extends AnalysisEngineImplBase impleme
       }
  
       finally {
-        UimaContextHolder.clearContext();
+        UimaContextHolder.setContext(prevContext);
         exitProcess();
       }
     }
