@@ -23,6 +23,7 @@ import org.apache.uima.UIMARuntimeException;
 
 /**
  * Runtime exception class for package org.apache.uima.cas.
+ * Messages in org.apache.uima.UIMAException_Messages
  */
 public class CASRuntimeException extends UIMARuntimeException {
 
@@ -126,6 +127,9 @@ public class CASRuntimeException extends UIMARuntimeException {
 	/** No sofaFS for specified sofaRef found. */
 	public static final String SOFAREF_NOT_FOUND = "SOFAREF_NOT_FOUND";
 
+  /** Sofa reference for FS {0} is required, but it is not set.  This can happen during deserialization when the type system changes where this FeatureStructure''s type definition is now a subtype of uima.cas.AnnotationBase but was not when the serialized form was created.*/
+  public static final String SOFAREF_NOT_SET= "SOFAREF_NOT_SET";
+
 	/** Can''t use standard set methods with SofaFS features. */
 	public static final String PROTECTED_SOFA_FEATURE = "PROTECTED_SOFA_FEATURE";
 
@@ -143,11 +147,17 @@ public class CASRuntimeException extends UIMARuntimeException {
    */
 	public static final String BLOB_SERIALIZATION = "BLOB_SERIALIZATION";
 
+	/** Unrecognized serialized CAS format. */
+  public static final String UNRECOGNIZED_SERIALIZED_CAS_FORMAT = "UNRECOGNIZED_SERIALIZED_CAS_FORMAT";
+	
 	/**
-   * Error trying to read BLOB data from an input stream and deserialize Stringo a CAS.
+   * Error while deserializing binary CAS. {0}.
    */
 	public static final String BLOB_DESERIALIZATION = "BLOB_DESERIALIZATION";
 
+	/** Deserializing Compressed Form 6 with CasLoadMode LENIENT, but no Type System provided. */
+	public static final String LENIENT_FORM_6_NO_TS = "LENIENT_FORM_6_NO_TS";
+	
 	/** Error trying to open a stream to Sofa data. */
 	public static final String SOFADATASTREAM_ERROR = "SOFADATASTREAM_ERROR";
 
@@ -206,6 +216,11 @@ public class CASRuntimeException extends UIMARuntimeException {
   /** While FS was in the index, illegal attempt to modify Feature "{0}" which is used as a key in one or more indexes; FS = "{1}" */
   public static final String ILLEGAL_FEAT_SET = "ILLEGAL_FEAT_SET";
 
+  /** Lenient deserialization not support for input of type {0}. */
+  public static final String LENIENT_NOT_SUPPORTED = "LENIENT_NOT_SUPPORTED";
+  
+  /** Multiply nested classloaders not supported.  Original base loader: {0}, current nested loader: {1}, trying to switch to loader: {2}.*/
+  public static final String SWITCH_CLASS_LOADER_NESTED = "SWITCH_CLASS_LOADER_NESTED";
   
 	public CASRuntimeException() {
 		super();
