@@ -59,7 +59,7 @@ public class CASMgrSerializer implements Serializable {
   // encoded explicitly. I.e., the fact that tokens are in any index that
   // annotations are in is implicit.
 
-  public int[] typeOrder;
+  public int[] typeOrder = null;
 
   /**
    * The index identifiers. Note that more than one identifier can refer to the same index.
@@ -107,7 +107,8 @@ public class CASMgrSerializer implements Serializable {
   // Type system encoding.
 
   /**
-   * A list of type names (symbol table). Note: numbering of types starts at <code>1</code>, and
+   * A list of type names (symbol table). fs-typed arrays have names XXXX[]. 
+   * Note: numbering of types starts at <code>1</code>, and
    * we index the names according to their internal code. That means that
    * <code>typeNames[0] == null</code>.
    */
@@ -510,4 +511,7 @@ public class CASMgrSerializer implements Serializable {
     return ir;
   }
 
+  public boolean hasIndexRepository() {
+    return this.typeOrder != null;
+  }
 }
