@@ -102,7 +102,7 @@ import org.apache.uima.cas.text.AnnotationFS;
  * <li>Call <code>getValueType</code> to find out whether the feature path evaluates to a String,
  * and int, a float, or their array counterparts.
  * <li>Depending on the leaf type, call the appropriate <code>evaluateAs </code> methods
- * </ol>
+ * </ul>
  * 
  */
 public class FeatureValuePathImpl implements FeatureValuePath {
@@ -808,17 +808,17 @@ public class FeatureValuePathImpl implements FeatureValuePath {
 		}
 		result.append(this.featureName);
 		if (this.isArrayOrList) {
-			result.append("[");
+			result.append('[');
 			if (!this.isBracketsOnly && this.arrayIndex >= 0) {
 				result.append(this.arrayIndex);
 			}
 			if (this.arrayIndex == LAST_ARRAY_ENTRY) {
 				result.append(LAST_ARRAY_ENTRY_MARKER);
 			}
-			result.append("]");
+			result.append(']');
 		}
 		if (this.childPath != null) {
-			result.append("/");
+			result.append('/');
 			result.append(this.childPath.toString());
 		}
 		return result.toString();
@@ -975,13 +975,13 @@ public class FeatureValuePathImpl implements FeatureValuePath {
 			if (this.isArrayOrList && (this.arrayIndex != USE_ALL_ENTRIES)) {
 				// in the case of, say, authornames[0], the feature is of type
 				// string array, but it will evaluate to a string.
-				this.valueTypeName = (String) CONTAINER_TO_ELEMENTYPE_MAP.get(type.getName());
+				this.valueTypeName = CONTAINER_TO_ELEMENTYPE_MAP.get(type.getName());
 			} else if (this.isListType) { // here, we can assume that
 				// arrayIndex =
 				// USE_ALL_ENTRIES
 				// we don't return lists, but arrays, so we need to map the type
 				// accordingly
-				this.valueTypeName = (String) LIST_TO_ARRAYTYPE_MAP.get(type.getName());
+				this.valueTypeName = LIST_TO_ARRAYTYPE_MAP.get(type.getName());
 			} else {
 				this.valueTypeName = SIMPLE_VAL_TYPES[Arrays.binarySearch(SIMPLE_VAL_TYPES, type.getName())];
 			}
