@@ -26,12 +26,13 @@ import org.apache.uima.UIMAFramework;
 import org.apache.uima.UIMA_IllegalStateException;
 import org.apache.uima.UimaContext;
 import org.apache.uima.UimaContextAdmin;
+import org.apache.uima.UimaContextHolder;
+import org.apache.uima.resource.impl.RelativePathResolver_impl;
 import org.apache.uima.resource.metadata.ResourceManagerConfiguration;
 import org.apache.uima.resource.metadata.ResourceMetaData;
 import org.apache.uima.util.InvalidXMLException;
 import org.apache.uima.util.Logger;
 import org.apache.uima.util.Settings;
-import org.apache.uima.util.impl.Settings_impl;
 
 /**
  * Implementation base class for {@link org.apache.uima.resource.Resource}s. Provides access to
@@ -82,32 +83,38 @@ public abstract class Resource_ImplBase implements Resource {
     if (aAdditionalParams != null) {
       mUimaContextAdmin = (UimaContextAdmin) aAdditionalParams.get(PARAM_UIMA_CONTEXT);
     }
-    if (mUimaContextAdmin == null) {// no, we have to create one    
-      // get or create ResourceManager
-      ResourceManager resMgr = null;
-      if (aAdditionalParams != null) {
-        resMgr = (ResourceManager) aAdditionalParams.get(PARAM_RESOURCE_MANAGER);
+    if (mUimaContextAdmin == null) {// no, we have to create one 
+    
+      // skip this part if initializing an external resource
+      // https://issues.apache.org/jira/browse/UIMA-5153
+      if (!(aSpecifier instanceof ConfigurableDataResourceSpecifier) &&
+          !(aSpecifier instanceof FileLanguageResourceSpecifier) &&
+          !(aSpecifier instanceof FileResourceSpecifier)) {
+        // get or create ResourceManager
+        ResourceManager resMgr = null;
+        if (aAdditionalParams != null) {
+          resMgr = (ResourceManager) aAdditionalParams.get(PARAM_RESOURCE_MANAGER);
+        }
+        if (resMgr == null) {
+          resMgr = UIMAFramework.newDefaultResourceManager();
+        }
+  
+        // get a Logger for this class and set its ResourceManager so that
+        // UIMA extension ClassLoader is used to locate message digests.
+        Logger logger = UIMAFramework.getLogger(this.getClass());
+        logger.setResourceManager(resMgr);
+        
+        ConfigurationManager configMgr = null;
+        if (aAdditionalParams != null) {
+          configMgr = (ConfigurationManager)aAdditionalParams.get(PARAM_CONFIG_MANAGER);
+        }
+        if (configMgr == null) {
+          configMgr = UIMAFramework.newConfigurationManager();
+        }
+  
+        // create and initialize UIMAContext
+        mUimaContextAdmin = UIMAFramework.newUimaContext(logger, resMgr, configMgr);
       }
-      if (resMgr == null) {
-        resMgr = UIMAFramework.newDefaultResourceManager();
-      }
-
-      // get a Logger for this class and set its ResourceManager so that
-      // UIMA extension ClassLoader is used to locate message digests.
-      Logger logger = UIMAFramework.getLogger(this.getClass());
-      logger.setResourceManager(resMgr);
-      
-      ConfigurationManager configMgr = null;
-      if (aAdditionalParams != null) {
-        configMgr = (ConfigurationManager)aAdditionalParams.get(PARAM_CONFIG_MANAGER);
-      }
-      if (configMgr == null) {
-        configMgr = UIMAFramework.newConfigurationManager();
-      }
-
-      // create and initialize UIMAContext
-      mUimaContextAdmin = UIMAFramework.newUimaContext(logger, resMgr, configMgr);
-
     } else {
       // configure logger of the UIMA context so that class-specific logging
       // levels and UIMA extension classLoader will work
@@ -155,7 +162,7 @@ public abstract class Resource_ImplBase implements Resource {
           }
         }
       }
-
+      
       // initialize configuration
       try {
         // createContext checks and skips repeated calls with same args (on different threads, for example)
@@ -169,29 +176,42 @@ public abstract class Resource_ImplBase implements Resource {
       }
 
       // initialize any external resource declared in this descriptor
+      // UIMA-5274  Set & restore the UimaContextHolder so that resources created on this thread can use the Settings
       ResourceManagerConfiguration resMgrCfg = ((ResourceCreationSpecifier) aSpecifier)
               .getResourceManagerConfiguration();
       if (resMgrCfg != null) {
+        UimaContext prevContext = UimaContextHolder.setContext(mUimaContextAdmin);
         try {
-          resMgrCfg.resolveImports(getResourceManager());
-        } catch (InvalidXMLException e) {
-          throw new ResourceInitializationException(e);
-        }
-        if (aAdditionalParams == null) {
+          try {
+            resMgrCfg.resolveImports(getResourceManager());
+          } catch (InvalidXMLException e) {
+            throw new ResourceInitializationException(e);
+          }
+          if (aAdditionalParams == null) {
             aAdditionalParams = new HashMap<String, Object>();
             aAdditionalParams.put(PARAM_RESOURCE_MANAGER, mUimaContextAdmin.getResourceManager());
-        } else {
-          if (!aAdditionalParams.containsKey(PARAM_RESOURCE_MANAGER)) {
-            // copy in case original is shared on multi-threads, or 
-            // is unmodifiable
-            // and to avoid updating passed - in map
-            aAdditionalParams = new HashMap<String, Object>(aAdditionalParams);
-            aAdditionalParams.put(PARAM_RESOURCE_MANAGER, mUimaContextAdmin.getResourceManager());
+          } else {
+            if (!aAdditionalParams.containsKey(PARAM_RESOURCE_MANAGER)) {
+              // copy in case original is shared on multi-threads, or
+              // is unmodifiable
+              // and to avoid updating passed - in map
+              aAdditionalParams = new HashMap<String, Object>(aAdditionalParams);
+              aAdditionalParams.put(PARAM_RESOURCE_MANAGER, mUimaContextAdmin.getResourceManager());
+            }
           }
+          // initializeExternalResources is synchronized
+
+          // https://issues.apache.org/jira/browse/UIMA-5153
+          final HashMap<String, Object> aAdditionalParmsForExtResources = new HashMap<String, Object>(aAdditionalParams); // copy in case
+          if (aAdditionalParmsForExtResources.get(PARAM_UIMA_CONTEXT) == null) {
+            aAdditionalParmsForExtResources.put(PARAM_UIMA_CONTEXT, mUimaContextAdmin);
+          }
+
+          mUimaContextAdmin.getResourceManager().initializeExternalResources(resMgrCfg,
+                  mUimaContextAdmin.getQualifiedContextName(), aAdditionalParmsForExtResources);
+        } finally {
+          UimaContextHolder.setContext(prevContext);
         }
-        // initializeExternalResources is synchronized
-        mUimaContextAdmin.getResourceManager().initializeExternalResources(resMgrCfg,
-                mUimaContextAdmin.getQualifiedContextName(), aAdditionalParams);
       }
 
       // resolve and validate this component's external resource dependencies
@@ -285,4 +305,16 @@ public abstract class Resource_ImplBase implements Resource {
   public CasManager getCasManager() {
     return getResourceManager().getCasManager();
   }
+  
+  public RelativePathResolver getRelativePathResolver(Map<String, Object> aAdditionalParams) {
+    RelativePathResolver relPathResolver = null;
+    if (aAdditionalParams != null) {
+      relPathResolver = (RelativePathResolver) aAdditionalParams.get(DataResource.PARAM_RELATIVE_PATH_RESOLVER);
+    }
+    if (relPathResolver == null) {
+      relPathResolver = new RelativePathResolver_impl();
+    }
+    return relPathResolver;
+  }
+
 }
