@@ -34,7 +34,8 @@ import java.util.Iterator;
 import java.util.Map;
 import java.util.Set;
 
-import junit.framework.Assert;
+import org.junit.Assert;
+
 import junit.framework.TestCase;
 
 import org.apache.uima.Constants;
@@ -63,6 +64,7 @@ import org.apache.uima.cas.text.AnnotationFS;
 import org.apache.uima.cas.text.AnnotationIndex;
 import org.apache.uima.examples.SourceDocumentInformation;
 import org.apache.uima.jcas.JCas;
+import org.apache.uima.resource.RelativePathResolver;
 import org.apache.uima.resource.Resource;
 import org.apache.uima.resource.ResourceInitializationException;
 import org.apache.uima.resource.ResourceManager;
@@ -87,6 +89,7 @@ import org.apache.uima.resource.metadata.impl.FsIndexKeyDescription_impl;
 import org.apache.uima.resource.metadata.impl.NameValuePair_impl;
 import org.apache.uima.resource.metadata.impl.TypePriorities_impl;
 import org.apache.uima.resource.metadata.impl.TypeSystemDescription_impl;
+import org.apache.uima.test.junit_extension.FileCompare;
 import org.apache.uima.test.junit_extension.JUnitExtension;
 import org.apache.uima.util.InvalidXMLException;
 import org.apache.uima.util.Level;
@@ -95,7 +98,7 @@ import org.apache.uima.util.XMLInputSource;
 import org.apache.uima.util.XMLParser;
 import org.apache.uima.util.XMLSerializer;
 import org.apache.uima.util.impl.ProcessTrace_impl;
-import org.apache.uima.util.impl.Settings_impl;
+import org.custommonkey.xmlunit.XMLAssert;
 import org.xml.sax.ContentHandler;
 
 /**
@@ -321,9 +324,15 @@ public class AnalysisEngine_implTest extends TestCase {
       ae.destroy();
 
       // descriptor with configuration parameter external overrides
-      // implicitly load settings values from the system property UimaExternalSettings
+      // implicitly load settings values from the 3 files in the system property UimaExternalOverrides
+      // Load 1st from filesystem, 2nd from classpath, and 3rd from datapath
+
       String resDir = "src/test/resources/TextAnalysisEngineImplTest/";
-      System.setProperty("UimaExternalOverrides", resDir+"testExternalOverride.settings,"+resDir+"testExternalOverride2.settings");
+      String prevDatapath = System.setProperty(RelativePathResolver.UIMA_DATAPATH_PROP, resDir);
+      System.setProperty("UimaExternalOverrides", 
+                      resDir+"testExternalOverride.settings," +
+                      "path:TextAnalysisEngineImplTest.testExternalOverride2," +
+                      "path:testExternalOverride4");
       in = new XMLInputSource(JUnitExtension.getFile("TextAnalysisEngineImplTest/AnnotatorWithExternalOverrides.xml"));
       desc = UIMAFramework.getXMLParser().parseAnalysisEngineDescription(in);
       ae1 = new PrimitiveAnalysisEngine_impl();
@@ -343,6 +352,11 @@ public class AnalysisEngine_implTest extends TestCase {
       Integer intValue = (Integer) ae1.getUimaContext().getConfigParameterValue("IntegerParam");
       Assert.assertEquals(43,  intValue.intValue());  // Will be 42 if external override not defined
       System.clearProperty("UimaExternalOverrides");
+      if (prevDatapath == null) {
+        System.clearProperty(RelativePathResolver.UIMA_DATAPATH_PROP);
+      } else {
+        System.setProperty(RelativePathResolver.UIMA_DATAPATH_PROP, prevDatapath);
+      }
       
       ae1.destroy();
       
@@ -362,7 +376,7 @@ public class AnalysisEngine_implTest extends TestCase {
       System.clearProperty("UimaExternalOverrides");
       
       // Same aggregate with invalid syntax for an array in the external overrides file
-      System.setProperty("UimaExternalOverrides", resDir+"testExternalOverride3.settings");
+      System.setProperty("UimaExternalOverrides", "file:"+resDir+"testExternalOverride3.settings");
       try {
         UIMAFramework.produceAnalysisEngine(desc);
         fail(); // should not get here
@@ -1570,6 +1584,7 @@ public class AnalysisEngine_implTest extends TestCase {
     // set the amount to a value which will show up if used
     // indent should not be used because we're using a parser mode which preserves
     // comments and ignorable white space.
+    // NOTE: Saxon appears to force the indent to be 3 - which is what the input file now uses.
     xmlSerializer.setOutputProperty("{http://xml.apache.org/xslt}indent-amount", "4");
     ContentHandler contentHandler = xmlSerializer.getContentHandler();
     contentHandler.startDocument();
@@ -1577,10 +1592,15 @@ public class AnalysisEngine_implTest extends TestCase {
     contentHandler.endDocument();
     os.close();
     
+    String inXml = FileCompare.file2String(inFile);
+    String cloneXml = FileCompare.file2String(cloneFile);
+    XMLAssert.assertXMLEqual(inXml,  cloneXml);
     // When building from a source distribution the descriptor may not have
     // appropriate line-ends so compute the length as if always 1 byte.
     int diff = fileLength(cloneFile) - fileLength(inFile);
     // One platform inserts a blank line and a final newline, so don't insist on perfection
+    // NOTE:  This fails with Saxon as it omits the xmlns attribute (why?) and omits the newlines between adjacent comments.
+    // It also produces many differences in indentation if the input is not indented by 3
     assertTrue("File size changed by "+diff+" should be no more than 2", diff >= -2 && diff <= 2);
 
     // Initialize all delegates and check the initialization order (should be declaration order)
