@@ -19,29 +19,31 @@
 
 package org.apache.uima.analysis_engine.impl;
 
-import java.io.BufferedReader;
 import java.io.ByteArrayInputStream;
+import java.io.File;
+import java.io.FileInputStream;
 import java.io.IOException;
 import java.io.InputStream;
-import java.io.InputStreamReader;
-import java.util.List;
-import java.util.Map.Entry;
-
-import junit.framework.Assert;
+import java.util.HashMap;
+import java.util.Map;
 
 import org.apache.uima.UIMAFramework;
 import org.apache.uima.UimaContext;
 import org.apache.uima.UimaContextHolder;
 import org.apache.uima.analysis_component.CasAnnotator_ImplBase;
+import org.apache.uima.analysis_engine.AnalysisEngineDescription;
 import org.apache.uima.analysis_engine.ResultSpecification;
 import org.apache.uima.cas.CAS;
 import org.apache.uima.cas.TypeSystem;
 import org.apache.uima.impl.UimaContext_ImplBase;
+import org.apache.uima.resource.Resource;
 import org.apache.uima.resource.ResourceConfigurationException;
 import org.apache.uima.resource.ResourceInitializationException;
+import org.apache.uima.test.junit_extension.JUnitExtension;
 import org.apache.uima.util.Settings;
 import org.apache.uima.util.UimaContextHolderTest;
-import org.apache.uima.util.impl.Settings_impl;
+import org.apache.uima.util.XMLInputSource;
+import org.junit.Assert;
 
 /**
  * Annotator class used for testing.
@@ -76,7 +78,7 @@ public class TestAnnotator2 extends CasAnnotator_ImplBase {
     // Note: this annotator launched with external overrides loaded from testExternalOverride2.settings 
     String contextName = ((UimaContext_ImplBase) aContext).getQualifiedContextName();
     if ("/ExternalOverrides/".equals(contextName)) {
-      String expected = "Context Holder Test";
+      // Test getting a (0-length) array of strings
       String[] actuals = null;
       try {
         actuals = UimaContextHolder.getContext().getSharedSettingArray("test.externalFloatArray");
@@ -85,17 +87,8 @@ public class TestAnnotator2 extends CasAnnotator_ImplBase {
       }
       Assert.assertEquals(0, actuals.length);
       
-      // prefix-suffix     Prefix-${suffix}
-      // suffix = should be ignored
-      String actual = null;
-      try {
-        actual = UimaContextHolder.getContext().getSharedSettingValue("context-holder");
-      } catch (ResourceConfigurationException e) {
-        Assert.fail(e.getMessage());
-      }
-      Assert.assertEquals(expected, actual);
-      
       // Test assigning an array to a string and vice-versa
+      String actual = null;
       try {
         actual = UimaContextHolder.getContext().getSharedSettingValue("test.externalFloatArray");
         Assert.fail("\"bad\" should create an error");
@@ -111,7 +104,12 @@ public class TestAnnotator2 extends CasAnnotator_ImplBase {
 
       // Test a stand-alone settings object
       Settings testSettings = UIMAFramework.getResourceSpecifierFactory().createSettings();
-      String lines = "foo = ${bar} \n bar : [ok \n OK] \n bad = ${missing}";
+      String lines = "foo = ${bar} \n" +
+                "bar : [ok \n OK] \n" +
+                "bad = ${missing} \n" +
+                "loop1 = one ${loop2} \n" +
+                "loop2 = two ${loop3} \n" +
+                "loop3 = three ${loop1} \n" ;
       InputStream is;
       try {
         is = new ByteArrayInputStream(lines.getBytes("UTF-8"));
@@ -125,11 +123,18 @@ public class TestAnnotator2 extends CasAnnotator_ImplBase {
         } catch (ResourceConfigurationException e) {
           System.err.println("Expected exception: " + e.toString());
         }
+        try {
+          val = testSettings.lookUp("loop2");
+          Assert.fail("\"loop2\" should create an error");
+        } catch (ResourceConfigurationException e) {
+          System.err.println("Expected exception: " + e.toString());
+        }
       } catch (Exception e) {
         Assert.fail(e.toString());
       }
       
       // Test POFO access via UimaContextHolder
+      String expected = "Context Holder Test";
       long threadId = Thread.currentThread().getId();
       UimaContextHolderTest testPojoAccess = new UimaContextHolderTest();
       try {
@@ -166,6 +171,39 @@ public class TestAnnotator2 extends CasAnnotator_ImplBase {
         Assert.fail();
       }
       
+      
+      // Test getting a string value
+      try {
+        actual = UimaContextHolder.getContext().getSharedSettingValue("context-holder");
+      } catch (ResourceConfigurationException e) {
+        Assert.fail(e.getMessage());
+      }
+      Assert.assertEquals(expected, actual);
+      
+      // Create a nested engine with a different settings
+      String resDir = "src/test/resources/TextAnalysisEngineImplTest/";
+      try {
+        //XMLInputSource in = new XMLInputSource(JUnitExtension.getFile("TextAnalysisEngineImplTest/AnnotatorWithExternalOverrides.xml"));
+        XMLInputSource in = new XMLInputSource(new File(resDir, "AnnotatorWithExternalOverrides.xml"));
+        AnalysisEngineDescription desc = UIMAFramework.getXMLParser().parseAnalysisEngineDescription(in);
+        Map<String, Object> additionalParams = new HashMap<String, Object>();
+        Settings extSettings = UIMAFramework.getResourceSpecifierFactory().createSettings();
+        FileInputStream fis = new FileInputStream(new File(resDir, "testExternalOverride2.settings"));
+        extSettings.load(fis);
+        fis.close();
+        additionalParams.put(Resource.PARAM_EXTERNAL_OVERRIDE_SETTINGS, extSettings);
+        UIMAFramework.produceAnalysisEngine(desc, additionalParams);
+      } catch (Exception e) {
+        Assert.fail();
+      }
+      
+      try {
+        actual = UimaContextHolder.getContext().getSharedSettingValue("context-holder");
+      } catch (ResourceConfigurationException e) {
+        Assert.fail(e.getMessage());
+      }
+      Assert.assertEquals(expected, actual);
+
     }
     // Used to check initialization order by testManyDelegates
     allContexts  = allContexts + contextName.substring(1);
