@@ -323,7 +323,8 @@ public class Jg {
 
     gui = new GUI(this);
     gui.addWindowListener(new WindowAdapter() {
-      public void windowClosing(WindowEvent e) {
+      @Override
+    public void windowClosing(WindowEvent e) {
         Prefs.set(gui);
         waiter.finished();
       }
@@ -717,10 +718,41 @@ public class Jg {
     try {
       typeDefinitionUri = new URI (td.getSourceUrlString());
     } catch (URISyntaxException e) {
-      return true; // may be overkill - but if td's source cant be parsed, ... likely out of project
+      return true; // may be overkill - but if td's source can't be parsed ... likely out of project
     }
     String tdPath = typeDefinitionUri.getPath();
-    boolean r = !tdPath.startsWith(projectDirPath);
+    
+    // Issue UIMA-4080 - If a type system resides in a JAR, then the path is null and it is
+    // certainly out of scope.
+    if (tdPath == null) {
+        return true;
+    }
+
+    // UIMA-4119 - On windows, the default path representation and the URI path representation
+    // differ: "/C:/..." vs. "C:\...". If the projectDirPath does not start with a /, we can be sure
+    // that it is not an absolute path in URI notation, so we try to resolve it.
+    // In this way, we can handle clients that use the URI notation (e.g. the Eclipse plugin)
+    // as well as clients that use file-system notation (e.g. jcasgen-maven-plugin or a simple
+    // invocation from the command line.
+    String resolvedProjectPath;
+    if (!projectDirPath.startsWith("/")) {
+        resolvedProjectPath = new File(projectDirPath).getAbsoluteFile().toURI().getPath();
+    }
+    else {
+        resolvedProjectPath = projectDirPath;
+    }
+    
+    // UIMA-4131 - Make sure that paths end in a slash so that "/my/project1" is not considered
+    // to be in the scope of "/my/project"
+    if (!tdPath.endsWith("/")) {
+        tdPath += "/";
+    }
+
+    if (!resolvedProjectPath.endsWith("/")) {
+        resolvedProjectPath += "/";
+    }
+
+    boolean r = !tdPath.startsWith(resolvedProjectPath);
     if (r) {
       return true;
     }
@@ -734,7 +766,7 @@ public class Jg {
           return true; //because a merged path is out of the project
         }
         String tempPath = tempURI.getPath();
-        if (!tempPath.startsWith(projectDirPath)) {
+        if (!tempPath.startsWith(resolvedProjectPath)) {
           return true; 
         }
       }
